<?php


/**
 * Class - Ajax
 */
class wps_ic_ajax extends wps_ic {

  public $options;
  public $api_url;


  public function __construct() {
    if (is_admin()) {

      $this->api_url = 'https://api.wpcompress.com/';

      $this->options = new wps_ic_options();

      // Bulk
      add_filter('bulk_actions-upload', array($this, 'register_bulk_action'));
      add_filter('handle_bulk_actions-upload', array($this, 'do_bulk_action'), 10, 3);

      // Check API status
      $this->add_ajax('wps_ic_check_api_status');

      // Load sample image for intro
      #$this->add_ajax('wps_ic_sample');

      // Window Unload
      $this->add_ajax('wps_ic_window_unload');

      // API Signup
      $this->add_ajax('wps_ic_api_signup');
      $this->add_ajax('wps_ic_api_connect');

      //
      $this->add_ajax('wps_ic_fixing');
      $this->add_ajax('wps_ic_clear_queue');

      // Compress Heartbeat
      $this->add_ajax('wps_ic_bulk_heartbeat');
      $this->add_ajax('wps_ic_compress_heartbeat_bulk');
      $this->add_ajax('wps_ic_compress_background_cron');

      // Compare Images
      $this->add_ajax('wps_ic_compare');

      // Change Setting
      $this->add_ajax('wps_ic_settings_change');

      // Create local backup
      $this->add_ajax('wps_ic_local_backup');

      // Compress the image - bulk
      $this->add_ajax('wps_ic_compress_single');
      $this->add_ajax('wps_ic_compress_bulk_prep_background');
      $this->add_ajax('wps_ic_compress_bulk_prep');
      $this->add_ajax('wps_ic_compress_background');
      $this->add_ajax('wps_ic_compress_bulk');
      $this->add_ajax('wps_ic_compress_bulk_stop');
      $this->add_ajax('wps_ic_compress_total_saved');

      // Compress the image - single
      $this->add_ajax('wps_ic_compress');

      // Restore the image - single
      $this->add_ajax('wps_ic_restore');
      $this->add_ajax('wps_ic_restore_attachment');

      // Authorize with remote API
      $this->add_ajax('wps_ic_authorize_api');

      // Deauthorize with remote API
      $this->add_ajax('wps_ic_deauthorize_api');

      // Exclude Image from Compress
      $this->add_ajax('wps_ic_pro_exclude');
    }
  }


  public function add_ajax($hook) {
    add_action('wp_ajax_' . $hook, array($this, $hook));
  }

  /**
   * Attachment exists?
   */
  public static function attachment_exists($attachment_id) {
    $attachment = get_post($attachment_id);
    if ( ! $attachment || get_post_type($attachment_id) !== 'attachment') {
      return false;
    }

    return true;
  }

  /**
   * Fixing stuck images
   * If image has meta in_bulk or has meta ic_data[old] but not ic_data[new]
   * but is not in queue table, restore image
   */
  public function wps_ic_fixing() {
    global $wps_ic, $wpdb;

    $attachments = get_posts(array('post_type' => 'attachment', 'posts_per_page' => - 1));
    if ($attachments) {
      foreach ($attachments as $attachment) {

        $in_bulk = get_post_meta($attachment->ID, 'wps_ic_in_bulk', true);
        $ic_data = get_post_meta($attachment->ID, 'wps_ic_data', true);

        // Prepare query
        $fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE attachment_ID=%s", $attachment->ID);
        $fetch = $wpdb->get_results($fetch);

        if ( ! $fetch) {
          if ( ! empty($in_bulk) || ( ! empty($ic_data['old']) && empty($ic_data['new']))) {
            // Not found in queue, fix it!
            delete_post_meta($attachment->ID, 'wps_ic_in_bulk');
            delete_post_meta($attachment->ID, 'wps_ic_data');
            #$wps_ic->compress->restore($attachment->ID, 'table');
          }
        }

      }
      // End it
      wp_send_json_error();
    }
  }

  /**
   * Exclude the image
   */
  public function wps_ic_pro_exclude() {
    global $wps_ic;
    $wps_ic->compress->exclude($_POST, 'html');
  }

  /**
   * Verify if API is online
   */
  public function wps_ic_check_api_status() {
    $call = wp_remote_get($this->api_url, array('timeout' => 5));
    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      if ($body->success == false) {
        wp_send_json_success();
      }
    }

    wp_send_json_error();
  }

  public function wps_ic_window_unload() {
    global $wps_ic;
    delete_option(WPS_IC_BULK);
    delete_option(WPS_IC_BULK_PROCESS);

    global $wpdb;

    $delete = $wpdb->prepare("DELETE FROM " . $wpdb->prefix . "ic_queue WHERE type=%s", 'compress_preview');
    $delete = $wpdb->query($delete);
  }

  public function wps_ic_clear_queue() {
    delete_option(WPS_IC_SAVINGS);
    delete_option(WPS_IC_BULK_PROCESS);
    delete_option(WPS_IC_BULK);
    delete_option(WPS_IC_CRON_QUEUE);
    delete_option(WPS_IC_WP_BULK_PROCESS);
    delete_option(WPS_IC_BULK_RESTORE);
  }

  public function wps_ic_api_connect() {
    global $wps_ic;
    $site = site_url();

    $apikey = sanitize_text_field($_POST['apikey']);

    // Setup URI
    $uri = 'http://keys.wpcompress.com/';
    $uri .= '?action=connect';
    $uri .= '&apikey=' . $apikey;
    $uri .= '&site=' . urlencode(site_url());

    // Verify API Key is our database and user has is confirmed getresponse
    $get = wp_remote_get($uri);

    if (wp_remote_retrieve_response_code($get) == 200) {
      $body = wp_remote_retrieve_body($get);
      $body = json_decode($body);

      if ($body->success == true && $body->data->api_key != '' && $body->data->response_key != '') {
        $wps_ic->options->set_option('api_key', $body->data->api_key);
        $wps_ic->options->set_option('response_key', $body->data->response_key);
        wp_send_json_success();
      }

      wp_send_json_error($body->data);
    }

    wp_send_json_error('0');
  }

  public function wps_ic_api_signup() {
    global $wps_ic;

    // Sanitize
    $email = sanitize_email($_POST['email']);
    $name  = sanitize_text_field($_POST['name']);
    $name  = urlencode($_POST['name']);

    // Is valid email
    if ( ! is_email($email)) {
      wp_send_json_error(array('code' => 100, 'msg' => 'Invalid e-mail!'));
    }

    $uri = 'http://keys.wpcompress.com/';
    $uri .= '?action=signup';
    $uri .= '&email=' . $email;
    $uri .= '&name=' . $name;
    $uri .= '&site=' . urlencode(site_url());

    $get = wp_remote_get($uri);

    if (wp_remote_retrieve_response_code($get) == 200) {
      $body = wp_remote_retrieve_body($get);
      $body = json_decode($body);

      if ($body->success == true) {

        $response = $body->data->code;

        if ($response == 110) {
          // Already subscribed and confirmed
          wp_send_json_error(array('code' => 110, 'msg' => 'You are already subscribed!<br/><a href="https://wpcompress.com/lost-api-key/" target="_blank">Retrieve API Key</a>'));
        } else {
          if ($response == 100) {
            // Subscribed but not confirmed
            wp_send_json_error(array('code' => 100, 'msg' => 'You have not confirmed your e-mail, we have resent confirmation!'));
          } else {
            if ($response == 111) {
              // Subscribed but was not in our database
              wp_send_json_error(array('code' => 111, 'msg' => 'You are already subscribed, but you were not found in our database!'));
            } else {
              if ($response == 200) {
                // Subscribed, waiting for confirmation
                wp_send_json_success(array('code' => 200, 'msg' => 'Thanks! Sent confirmation!'));
              } else {
                if ($response == 202) {
                  // Subscribed, waiting for confirmation
                  wp_send_json_error(array('code' => 202, 'msg' => 'You are already subscribed and confirmed!<br/><a href="https://wpcompress.com/lost-api-key/" target="_blank">Retrieve API Key</a>'));
                } else {
                  // Subscribed, waiting for confirmation
                  wp_send_json_error(array('code' => 202, 'msg' => 'Unkown error occured!'));
                }
              }
            }
          }
        }

      }

      // Uknown Error
      wp_send_json_error(array('code' => 0));
    }

    wp_send_json_error(array('code' => 0));
  }

  public function wps_ic_bulk_heartbeat() {
    global $wps_ic;

    $default                   = array();
    $default['compress_queue'] = '';
    $default['restore_queue']  = '';

    // Change memory limits
    $wps_ic->backup->change_limits();

    // Set status to 1, kill other heartbeats and actions
    $status = get_option(WPS_IC_STATUS);

    if ($status) {
      wp_send_json_success($default);
    }

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    $compress_queue = $wps_ic->queue->get_compress_queue();
    $restore_queue  = $wps_ic->queue->get_restore_queue();

    $uploaddir  = wp_upload_dir();
    $uploadfile = $uploaddir['path'] . '/';

    // Cron do compress
    if ( ! empty($compress_queue)) {
      $default['compress_queue'] = $compress_queue;
      wp_send_json_success($default);
    }

    // Cron do restore
    if ( ! empty($restore_queue)) {
      $default['restore_queue'] = $restore_queue;
      wp_send_json_success($default);
    }

    wp_send_json_error($default);
  }

  public function wps_ic_compress_heartbeat_bulk() {
    global $wps_ic, $wpdb;

    $default                       = array();
    $default['bulk_restore_queue'] = '';
    $default['bulk_queue']         = '';

    // Set status to 1, kill other heartbeats and actions
    $status = get_option(WPS_IC_STATUS);

    if ($status) {
      wp_send_json_success($default);
    }

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    // Grab all files for compression
    $files = get_option(WPS_IC_BULK);

    // Grab last known status of files
    $status = get_option(WPS_IC_BULK_PROCESS);

    $uploaddir  = wp_upload_dir();
    $uploadfile = $uploaddir['path'] . '/';

    // Cron fetch thumbnails
    if ($files) {
      // Set status to 1, kill other heartbeats and actions
      update_option(WPS_IC_STATUS, '1');
      wp_send_json_success(array('files' => $files));
    }

    wp_send_json_error(array('files' => '0'));
  }

  /**
   * Compare two images
   */
  public function wps_ic_compare() {
    global $wps_ic, $wpdb;

    $original = sanitize_text_field($_POST['original']);

    // Get original from backup
    $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $original);
    $backup = $wpdb->get_results($backup);

    if (empty($backup)) {
      // Backup does not exist, file not compressed.
      $before_data = get_attached_file($original);
      $before      = wp_get_attachment_image_src($original, 'full');
      $before      = $before[0];
      $before_size = filesize($before_data);
      // Compress file on fly
      $after_data = $wps_ic->compress->preview(array('attachment_id' => $original));
      $after      = $after_data['uri'];
      $after_size = $after_data['size'];
    } else {
      // File compressed
      $before      = $backup[0]->url;
      $before_size = filesize($backup[0]->path);
      // Compress file fetch
      $after_data = get_attached_file($original);
      $after      = wp_get_attachment_image_src($original, 'full');
      $after      = $after[0];
      $after_size = filesize($after_data);
    }

    $output = '';
    $output .= '<div class="wps-ic-compare">
					  <div class="half">
                        <img src="' . $before . '"/>
                        <h3>Before</h3>
                        <h3>' . size_format($before_size, 2) . '</h3>
                      </div>
                      <div class="half">
                        <img src="' . $after . '"/>
                        <h3>After</h3>
                        <h3>' . size_format($after_size, 2) . '</h3>
                      </div>
                      </div>';

    wp_send_json_success($output);
  }

  /**
   * Change Settings Value
   */
  public function wps_ic_settings_change() {
    global $wps_ic;

    $what  = sanitize_text_field($_POST['what']);
    $value = sanitize_text_field($_POST['value']);

    $settings = $wps_ic->options->get_settings();

    $settings[ $what ] = $value;
    update_option(WPS_IC_SETTINGS, $settings);

    wp_send_json_success();
  }

  /**
   * Register bulk action
   */
  public function register_bulk_action($bulk_actions) {
    if ($this->options->get_option('response_key') != '') {
      #$bulk_actions['compress'] = 'Compress';
      $bulk_actions['restore'] = 'Restore';
    }

    return $bulk_actions;
  }

  /**
   * Bulk Compress
   */
  public function do_bulk_action($redirect_to, $action_name, $post_ids) {
    global $wps_ic, $wpdb;

    if ('compress' == $action_name) {

      $bulk_queue = get_option(WPS_IC_BULK);

      foreach ($post_ids as $post_id) {
        $type               = get_post_mime_type($post_id);
        $already_compressed = get_post_meta($post_id, 'wps_ic_compressed', true);

        // Is already compressed?
        $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $post_id);
        $backup = $wpdb->get_results($backup);

        if ($backup || $already_compressed) {
          continue;
        }

        if ($type != 'image/jpeg' && $type != 'image/png' && $type != 'image/gif' && $type != 'image/jpg') {
          continue;
        }

        $excluded = get_post_meta($post_id, 'wps_ic_exclude', true);
        if ( ! empty($excluded)) {
          continue;
        }

        // Fetch already queued
        $compress_queue = get_option(WPS_IC_CRON_QUEUE);

        // Sanitize data
        $_POST = $this->sanitize_post($_POST);

        // Set compressing flags
        update_post_meta($post_id, 'wps_ic_compressing', 'true');

        $compress_queue[ $post_id ] = $post_id;

        // Update compress queue
        update_option(WPS_IC_CRON_QUEUE, $compress_queue);

        if (empty($already_compressed) || $already_compressed == 'false') {

          // Add image to bulk
          $bulk_queue[ $post_id ] = $post_id;

          update_post_meta($post_id, 'wps_ic_in_bulk', 'true');

          #$image = wp_get_attachment_image_src($post_id, 'full');
          #$wps_ic->compress->single_full(array('attachment_id' => $post_id), 'none');
        }
      }

      update_option(WPS_IC_BULK, $bulk_queue);

      $redirect_to = add_query_arg('bulk_posts_processed', count($post_ids), $redirect_to);

      return $redirect_to;
    } else {
      if ('restore' == $action_name) {

        foreach ($post_ids as $post_id) {

          $excluded = get_post_meta($post_id, 'wps_ic_exclude', true);
          if ( ! empty($excluded) && $excluded == 'true') {
            continue;
          }

          // Fetch backup log
          $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $post_id);
          $backup = $wpdb->get_results($backup);

          if ( ! empty($backup[0])) {

            // Add to Queue
            $wps_ic->queue->add_queue($post_id, 'restore');

            // Mark as processing
            update_post_meta($post_id, 'wps_ic_in_bulk', 'true');

          }
        }

        $redirect_to = add_query_arg('other_bulk_posts_precessed', count($post_ids), $redirect_to);

        return $redirect_to;
      }
    }
  }

  /**
   * Sanitizes POST variables
   *
   * @param $post
   *
   * @return mixed
   */
  public function sanitize_post($post) {
    if ($post) {

      foreach ($post as $key => $value) {
        $post[ $key ] = sanitize_text_field($value);
      }
    }

    return $post;
  }

  /**
   * Total Saved Bulk
   */
  public function wps_ic_compress_total_saved() {
    global $wps_ic, $wpdb;

    $output  = '';
    $stats   = $wps_ic->stats->get();
    $savings = $stats['bytes'];

    if ($savings) {
      wp_send_json_success(array('output' => '', 'saved' => $savings));
    }

    wp_send_json_error();
  }

  /**
   * Prep Compress the images BULK!!
   */
  public function wps_ic_compress_bulk_prep() {
    global $wps_ic,$wpdb;
    $settings = get_option(WPS_IC_SETTINGS);

    $output      = array();
    $count_files = 0;

    delete_option(WPS_IC_STATS_BULK_FILES);
    delete_option(WPS_IC_STATS_BULK_TOTAL_FILES);
    delete_option(WPS_IC_STATS_BULK_SAVINGS);
    delete_option(WPS_IC_STATS_BULK_AVG);

    $attachments = get_posts(array('post_type' => 'attachment', 'posts_per_page' => '-1', 'post_status' => 'any'));

    if ($attachments) {
      foreach ($attachments as $attachment) {

        $excluded = get_post_meta($attachment->ID, 'wps_ic_exclude', true);
        if ( ! empty($excluded)) {
          continue;
        }

        $data = get_post_meta($attachment->ID, 'wps_ic_data', true);
        if ( ! empty($data) && ! empty($data['new'])) {
          continue;
        }

        if ($data == 'not_able') {
          $queued_excluded[] = $attachment->ID;
          continue;
        }

        $file_data = get_attached_file($attachment->ID);
        $type      = wp_check_filetype($file_data);

        $allowed_types         = array();
        $allowed_types['jpg']  = 'jpg';
        $allowed_types['jpeg'] = 'jpeg';
        $allowed_types['gif']  = 'gif';
        $allowed_types['png']  = 'png';

        if ( ! in_array($type['ext'], $allowed_types)) {
          continue;
        }

        // Is already compressed?
        $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attachment->ID);

        $backup = $wpdb->get_results($backup);

        if (empty($backup)) {

          // First Create Backup
          if ($settings['backup'] == '1') {
            $wps_ic->backup->local_backup($attachment->ID);
          }

        }

        // Count
        $count_files ++;

        // Add queue
        $wps_ic->queue->add_queue($attachment->ID, 'compress_preview');
      }

      update_option(WPS_IC_STATS_BULK_TOTAL_FILES, $count_files);

      $output = $wps_ic->queue->get_compress_preview_queue();
      wp_send_json_success($output);
    }

    wp_send_json_error();
  }

  /**
   * BACKGROUND Process Image
   */
  public function wps_ic_compress_background_cron() {
    global $wps_ic;

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    $response         = array();
    $compressed_items = get_option('wps_ic_compressed');

    if ( ! empty($compressed_items)) {

      if ( ! empty($compressed_items['waiting'])) {
        foreach ($compressed_items['waiting'] as $key => $attachment_ID) {

          // Saved meta data
          $meta = get_post_meta($attachment_ID, 'wps_ic_data', true);

          $time = get_post_meta($attachment_ID, 'wps_ic_times', true);

          $client   = ($time['client_ended'] - $time['client_started']);
          $compress = ($time['compress_ended'] - $time['compress_started']);
          $time     = number_format($client + $compress, 1) . 's';

          // Compressed
          $file_data            = get_attached_file($attachment_ID);
          $compressed_value_num = $compressed_value = filesize($file_data);
          $compressed_value     = size_format($compressed_value, 2);

          // Old size
          $uncompressed_value_num = $meta['old']['size'] / 1000;
          $uncompressed_value     = size_format($meta['old']['size'], 2);

          // Saved total
          $saved_total = $meta['old']['size'] - $compressed_value_num;
          $saved_total = size_format($saved_total, 2);

          // Saved
          $saved = ($compressed_value_num / ($uncompressed_value_num * 1000)) * 100;
          $saved = round(100 - $saved, 0) . '%';

          // Dimensions
          if (empty($meta['old']['data']['width'])) {
            $attachment_dimensions_old = 'Unreadable';
            $attachment_dimensions_new = 'Unreadable';
          } else {
            $attachment_dimensions_old = $meta['old']['data']['width'] . 'x' . $meta['old']['data']['height'];
            $attachment_dimensions_new = $meta['new']['data']['width'] . 'x' . $meta['new']['data']['height'];
          }

          $output = '';
          $output .= '<div class="wps-ic-compressed">';
          $output .= '<img src="' . $logo_compressed . '" />';
          $output .= '<h5>' . $saved . '</h5>';
          $output .= '<h5>' . $saved_total . '</h5>';
          $output .= '<h5>Saved</h5>';
          $output .= '</div>';

          $actions = '';

          $actions .= '<div class="wps-ic-media-actions">';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';

          $actions .= '<li class="wps-ic-weight"><span class="strike">' . $uncompressed_value . '</span></li>';
          $actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '</li>';

          $actions .= '<li class="wps-ic-size-compressed"><span>' . $compressed_value . '</span></li>';
          #$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '<div class="wps-ic-media-actions-toolbox">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '<li class="wps-ic-action">';

          $actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-image_id="' . $attachment_ID . '">Restore</button>';

          $actions .= '</li>';
          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '</div>';

          $response[ $attachment_ID ]['output']  = $output;
          $response[ $attachment_ID ]['actions'] = $actions;

          unset($compressed_items['waiting'][ $key ]);
          update_option('wps_ic_compressed', $compressed_items);
        }
        wp_send_json_success($response);
      }
    }
    wp_send_json_error();
  }

  /**
   * Bulk Compress Prepare
   */
  public function wps_ic_compress_bulk_prep_background() {
    global $wps_ic,$wpdb;
    $settings = get_option(WPS_IC_SETTINGS);

    $output = array();

    $count_files = 0;

    delete_option(WPS_IC_STATS_BULK_FILES);
    delete_option(WPS_IC_STATS_BULK_TOTAL_FILES);
    delete_option(WPS_IC_STATS_BULK_SAVINGS);
    delete_option(WPS_IC_STATS_BULK_AVG);

    // Clear whole queue
    $wps_ic->queue->clear_queue('compress_bulk');

    /*
     * Check if it's a single file clicked or bulk compress button
     */
    $attachments = get_posts(array('post_type' => 'attachment', 'posts_per_page' => '-1', 'post_status' => 'any'));

    if ($attachments) {
      foreach ($attachments as $attachment) {

        $compressed = get_post_meta($attachment->ID, 'wps_ic_compressed', true);
        if ( ! empty($compressed) && $compressed == 'true') {
          #continue;
        }

        $excluded = get_post_meta($attachment->ID, 'wps_ic_exclude', true);
        if ( ! empty($excluded) && $excluded == 'true') {
          continue;
        }

        $data = get_post_meta($attachment->ID, 'wps_ic_data', true);
        if ( ! empty($data) && !empty($data['new'])) {
          continue;
        }

        if ($data == 'not_able') {
          continue;
        }

        // Allowed type
        $file_data = get_attached_file($attachment->ID);
        $type      = wp_check_filetype($file_data);

        $allowed_types         = array();
        $allowed_types['jpg']  = 'jpg';
        $allowed_types['jpeg'] = 'jpeg';
        $allowed_types['gif']  = 'gif';
        $allowed_types['png']  = 'png';

        if ( ! in_array($type['ext'], $allowed_types)) {
          continue;
        }

        // Update time
        $times                   = get_post_meta($attachment->ID, 'wps_ic_times', true);
        $times['client_started'] = microtime(true);
        update_post_meta($attachment->ID, 'wps_ic_times', $times);

        // Old image data
        $file_data                    = get_attached_file($attachment->ID);
        $compress_data['old']['size'] = filesize($file_data);
        $compress_data['old']['data'] = wp_get_attachment_metadata($attachment->ID);
        update_post_meta($attachment->ID, 'wps_ic_data', $compress_data);

        // Get image ID and SRC
        $image                     = wp_get_attachment_image_src($attachment->ID, 'full');
        $output[ $attachment->ID ] = array('attachment_id' => $attachment->ID, 'image' => $image[0]);

        // Set compressing
        update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

        // Update time
        $times                 = get_post_meta($attachment->ID, 'wps_ic_times', true);
        $times['client_ended'] = microtime(true);
        update_post_meta($attachment->ID, 'wps_ic_times', $times);

        // Is already compressed?
        $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attachment->ID);

        $backup = $wpdb->get_results($backup);

        if (empty($backup)) {

          // First Create Backup
          if ($settings['backup'] == '1') {
            $wps_ic->backup->local_backup($attachment->ID);
          }

        }

        // Add queue
        $queue = $wps_ic->queue->add_queue($attachment->ID, 'compress_bulk');

        if ($queue) {
          // Count files
          $count_files ++;
        }
      }

      update_option(WPS_IC_STATS_BULK_TOTAL_FILES, $count_files);

      if (empty($_POST['attachment_id'])) {
        $queue = $wps_ic->queue->get_compress_queue();
        wp_send_json_success($queue);
      } else {
        wp_send_json_success();
      }
    }

    wp_send_json_error();
  }

  /**
   * Compress Single Image
   */
  public function wps_ic_compress_single() {
    global $wps_ic;

    $output = array();
    $output = get_option(WPS_IC_BULK);

    // Change memory limits
    $wps_ic->backup->change_limits();

    /*
     * Check if it's a single file clicked or bulk compress button
     */
    $queued          = array();
    $queued_excluded = array();

    if ( ! empty($_POST['attachments'])) {
      $attachments = $_POST['attachments'];
      $attachments = (object)$attachments;

      $attachments_output = array();
      foreach ($attachments as $id => $attachment) {
        $attachments_output[] = (object)array('ID' => $attachment);
      }

      $attachments = (object)$attachments_output;

    } else {
      $attachments[] = (object)array('ID' => $_POST['attachment_id']);
      $attachments   = (object)$attachments;
    }

    if ($attachments) {
      foreach ($attachments as $attachment) {

        $compressed = get_post_meta($attachment->ID, 'wps_ic_compressed', true);
        if ( ! empty($compressed) && $compressed == 'true') {
          $queued_excluded[] = $attachment->ID;
          continue;
        }

        $excluded = get_post_meta($attachment->ID, 'wps_ic_exclude', true);
        if ( ! empty($excluded) && $excluded == 'true') {
          $queued_excluded[] = $attachment->ID;
          continue;
        }

        $data = get_post_meta($attachment->ID, 'wps_ic_data', true);
        if ( ! empty($data) && ! empty($data['new'])) {
          $queued_excluded[] = $attachment->ID;
          continue;
        }

        if ($data == 'not_able') {
          $queued_excluded[] = $attachment->ID;
          continue;
        }


        // Update time
        $times                   = get_post_meta($attachment->ID, 'wps_ic_times', true);
        $times['client_started'] = microtime(true);

        // Set compressing
        update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

        // Add to Queue
        $wps_ic->queue->add_queue($attachment->ID, 'compress');
        $queued[] = $attachment->ID;

        // Update time
        $times['client_ended'] = microtime(true);
        update_post_meta($attachment->ID, 'wps_ic_times', $times);
      }

      if (empty($_POST['attachment_id'])) {
        wp_send_json_success(admin_url('upload.php'));
      } else {
        wp_send_json_success(array('list' => $attachments, 'queued' => $queued, 'queue_excluded' => $queued_excluded));
      }
    }

    wp_send_json_error();
  }

  /**
   * Compress Bulk STOP
   */
  public function wps_ic_compress_bulk_stop() {
    global $wps_ic;

    $output = '';

    delete_option(WPS_IC_SAVINGS);
    delete_option(WPS_IC_BULK_PROCESS);
    delete_option(WPS_IC_BULK_BACKGROUND);
    delete_option(WPS_IC_BULK_BACKGROUND_THUMBNAILS);
    delete_option(WPS_IC_BULK_RESTORE);
    delete_option(WPS_IC_BULK_DONE);

    $queue = $wps_ic->queue->get_compress_bulk_queue();
    if ($queue) {
      foreach ($queue as $attachment_ID => $values) {
        delete_post_meta($attachment_ID, 'wps_ic_data');
        delete_post_meta($attachment_ID, 'wps_ic_times');
        delete_post_meta($attachment_ID, 'wps_ic_in_bulk');
        $wps_ic->queue->remove_queue($attachment_ID);
      }
    }

    $output .= '<h3>You have stopped compression...</h3>';

    wp_send_json_success($output);
  }

  /**
   * Compress the images background
   */
  public function wps_ic_compress_background() {
    global $wps_ic, $wpdb;
    $bulk_list = $wps_ic->queue->get_compress_bulk_queue();

    if ($bulk_list) {

      foreach ($bulk_list as $attachment_id => $values) {
        // Update time
        $times                   = get_post_meta($attachment_id, 'wps_ic_times', true);
        $times['client_started'] = microtime(true);
        update_post_meta($attachment_id, 'wps_ic_times', $times);

        // Compress Data
        $file_data                    = get_attached_file($attachment_id);
        $compress_data['old']['size'] = filesize($file_data);
        $compress_data['old']['data'] = wp_get_attachment_metadata($attachment_id);
        update_post_meta($attachment_id, 'wps_ic_data', $compress_data);

        $response = $wps_ic->compress->single_bulk(array('attachment_id' => $attachment_id, 'data' => $values['status']), 'html');
        wp_send_json_success($response);
      }
    } else {
      wp_send_json_error('No queue!');
    }
  }

  /**
   * Compress the images BULK!!
   */
  public function wps_ic_compress_bulk() {
    global $wps_ic, $wpdb;

    $bulk_list = $wps_ic->queue->get_compress_preview_queue();

    if ($bulk_list) {
      foreach ($bulk_list as $attachment_id => $values) {
        // Update time
        $times                   = get_post_meta($attachment_id, 'wps_ic_times', true);
        $times['client_started'] = microtime(true);
        update_post_meta($attachment_id, 'wps_ic_times', $times);

        // Compress Data
        $file_data                    = get_attached_file($attachment_id);
        $compress_data['old']['size'] = filesize($file_data);
        $compress_data['old']['data'] = wp_get_attachment_metadata($attachment_id);
        update_post_meta($attachment_id, 'wps_ic_data', $compress_data);

        $response = $wps_ic->compress->single_bulk(array('attachment_id' => $attachment_id), 'html');
        wp_send_json_success($response);
      }
    } else {
      wp_send_json_error();
    }
  }

  /**
   * Compress the image
   */
  public function wps_ic_compress() {
    global $wps_ic, $wpdb;

    // Change memory limits
    $wps_ic->backup->change_limits();

    // Sanitize data
    $_POST = $this->sanitize_post($_POST);

    // Get attachment ID
    $attachment_id = $_POST['attachment_id'];

    // attachment already done?
    $query = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE attachment_ID=%s", $attachment_id);
    $query = $wpdb->get_results($query);

    if ( ! $query || empty($query) || empty($query[0])) {
      wp_send_json_error('Done already!');
    } else {
      $wps_ic->queue->remove_queue($attachment_id);
    }

    // Set compressing flags
    update_post_meta($attachment_id, 'wps_ic_compressing', 'true');
    update_post_meta($attachment_id, 'wps_ic_in_bulk', 'true');

    // Update time
    $times                   = get_post_meta($attachment_id, 'wps_ic_times', true);
    $times['client_started'] = microtime(true);
    update_post_meta($attachment_id, 'wps_ic_times', $times);

    // Run Ajax for full image
    $wps_ic->compress->single_bulk(array('attachment_id' => $attachment_id), 'table');

    wp_send_json_error('Did not run compress!');
  }

  /**
   * Restore the file
   */
  public function wps_ic_restore_attachment() {
    global $wps_ic;

    // Change memory limits
    $wps_ic->backup->change_limits();

    $attachment_ID = sanitize_text_field($_POST['attachment_id']);

    /*if ( ! self::attachment_exists($attachment_ID)) {
      $wps_ic->queue->remove_queue($attachment_ID);
      wp_send_json_error();
    }*/

    $wps_ic->queue->remove_queue($attachment_ID);
    $wps_ic->compress->restore($attachment_ID, 'table');
  }

  /**
   * Restore the image
   */
  public function wps_ic_restore() {
    global $wps_ic;

    // Change memory limits
    $wps_ic->backup->change_limits();

    $output        = array();
    $attachment_ID = sanitize_text_field($_POST['attachment_id']);

    /*
     * Check if it's a single file clicked or bulk compress button
     */
    $queued          = array();
    $queued_excluded = array();

    if ( ! empty($_POST['attachments'])) {
      $attachments = $_POST['attachments'];
      $attachments = (object)$attachments;

      $attachments_output = array();
      foreach ($attachments as $id => $attachment) {
        $attachments_output[] = (object)array('ID' => $attachment);
      }

      $attachments = (object)$attachments_output;

    } else {
      $attachments[] = (object)array('ID' => $_POST['attachment_id']);
      $attachments   = (object)$attachments;
    }

    if ($attachments) {
      foreach ($attachments as $attachment) {

        // Set compressing
        update_post_meta($attachment->ID, 'wps_ic_restoring', 'true');
        update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

        // Add to Queue
        $wps_ic->queue->add_queue($attachment->ID, 'restore');
        $queued[] = $attachment->ID;

      }

      wp_send_json_success(array('list' => $attachments, 'queued' => $queued));
    }

    wp_send_json_error();
  }

  /**
   * Create Local backup
   */
  public function wps_ic_local_backup() {
   # global $wps_ic;
    #$_POST = $this->sanitize_post($_POST);
    #$wps_ic->backup->local_backup($_POST['attachment_id']);
  }


  /**
   * Authorize site with remote api
   */
  public function wps_ic_authorize_api() {
    global $wps_ic;

    $site    = site_url();
    $api_key = $this->options->get_option('api_key');

    $call = wp_remote_get($this->api_url . '?authorize=true&site=' . urlencode($site) . '&api_key=' . $api_key);

    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      $wps_ic->options->set_option('response_key', $body->data);
      wp_send_json_success();
    }

    wp_send_json_error('Oops! There has been an error in authorize api!');
  }


  /**
   * Deauthorize site with remote api
   */
  public function wps_ic_deauthorize_api() {
    global $wps_ic;

    $site    = site_url();
    $api_key = $this->options->get_option('api_key');

    $call = wp_remote_get($this->api_url . '?deauthorize=true&site=' . urlencode($site) . '&api_key=' . $api_key);

    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      $wps_ic->options->set_option('response_key', '');
      wp_send_json_success();
    }

    wp_send_json_error('Oops! There has been an error in authorize api!');
  }


}