<?php


/**
 * Class - Compress
 */
class wps_ic_compress extends wps_ic {


  public function __construct() {
    $this->api_url = 'https://api.wpcompress.com/';
    #$this->api_url = 'http://138.197.111.233/';
    #$this->api_url = 'http://209.59.129.48/~wpcompress/';
    add_action('delete_attachment', array(__CLASS__, 'delete_attachment'));
  }


  public static function delete_attachment($post_id) {
    global $wpdb, $wps_ic;

    // Remove from Queue
    $background_process = get_option(WPS_IC_BULK_BACKGROUND);
    unset($background_process[ $post_id ]);
    update_option(WPS_IC_BULK_BACKGROUND, $background_process);

    // Remove thumbnails from Queue
    $background_thumbnails = get_option(WPS_IC_BULK_BACKGROUND_THUMBNAILS);
    unset($background_thumbnails[ $post_id ]);
    update_option(WPS_IC_BULK_BACKGROUND_THUMBNAILS, $background_thumbnails);

    // Remove from queue
    $compress_queue = get_option(WPS_IC_CRON_QUEUE);
    unset($compress_queue[ $post_id ]);
    update_option(WPS_IC_CRON_QUEUE, $compress_queue);

    // Remove from bulk restore queue
    $restore_bulk = get_option(WPS_IC_BULK_RESTORE);
    unset($restore_bulk[ $post_id ]);
    update_option(WPS_IC_BULK_RESTORE, $restore_bulk);

    // Remove stats
    $wpdb->delete($wpdb->prefix . 'ic_compressed', array('attachment_ID' => $post_id));
    $wpdb->delete($wpdb->prefix . 'ic_queue', array('attachment_ID' => $post_id));

    // Delete backup
    $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $post_id);
    $backup = $wpdb->get_results($backup);
    // Does backup exist?
    if ($backup) {
      foreach ($backup as $backup_file) {
        $last_backup_filename = $backup[0]->filename;
        unlink($wps_ic->backup->local_backup_path . $last_backup_filename);
      }
    }

    $backup = $wpdb->query("DELETE FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s", $post_id);
    $query  = $wpdb->query("DELETE FROM " . $wpdb->prefix . "ic_compressed WHERE attachment_ID='" . $post_id . "'");
  }


  public function restore($attachment_id, $response = 'json') {
    global $wps_ic, $wpdb;

    // Change memory limits
    $wps_ic->backup->change_limits();

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    $icon_compress = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress.png" alt="Compress" />';
    $icon_pro      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro.png" alt="Compare with Pro" />';
    $icon_compare  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare.png" alt="Compare" />';
    $icon_restore  = '<img src="' . WPS_IC_URI . 'assets/images/icon-restore.png" alt="Restore" />';

    $icon_compress_disabled = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress-disabled.png" alt="Compress Disabled" />';
    $icon_pro_disabled      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro-disabled.png" alt="Compare with Pro Disabled" />';
    $icon_compare_disabled  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-disabled.png" alt="Compare Disabled" />';

    $wps_ic->backup->change_limits();

    $compress_data = get_post_meta($attachment_id, 'wps_ic_data', true);
    $file_data     = get_attached_file($attachment_id);

    if ( ! empty($file_data)) {

      // File path
      $file_name = basename($file_data);
      $file_path = str_replace($file_name, '', $file_data);

      // Fetch backup log
      $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attachment_id);
      $backup = $wpdb->get_results($backup);

      if ( ! empty($backup)) {

        clearstatcache();

        // Fetch last backup
        $last_backup_filename = $backup[0]->filename;

        // Copy the actual file
        unlink($file_path . $file_name);
        $copy = $wps_ic->backup->copy_file($wps_ic->backup->local_backup_path . $last_backup_filename, $file_path . $file_name);

        // Delete backup record
        $wpdb->query("DELETE FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID='" . $attachment_id . "'");

        #$wps_ic->queue->remove_compress_queue($attachment_id);

        // Run thumbnail regeneration
        wp_schedule_single_event(time(), 'wps_ic_single_thumbnail', $attachment_id);

        // Delete backup file
        unlink($wps_ic->backup->local_backup_path . $last_backup_filename);

        /// Add file to compressed table for stats
        $wpdb->delete($wpdb->prefix . 'ic_compressed', array('attachment_ID' => $attachment_id));

        // Generate attachment metadata
        $fullsizepath = get_attached_file($attachment_id);
        #$attach_data  = wp_generate_attachment_metadata($attachment_id, $fullsizepath);

        $attach_data = wp_generate_attachment_metadata($attachment_id, $fullsizepath);
        wp_update_attachment_metadata($attachment_id, $attach_data);

        // Delete compress data
        delete_post_meta($attachment_id, 'wps_ic_times');
        delete_post_meta($attachment_id, 'wps_ic_compressed');
        delete_post_meta($attachment_id, 'wps_ic_data');
        delete_post_meta($attachment_id, 'wps_ic_in_bulk');
        delete_post_meta($attachment_id, 'wps_ic_compressing');
        delete_post_meta($attachment_id, 'wps_ic_restoring');

        // Data
        $file_data                         = get_attached_file($attachment_id);
        $compress_data['original']['size'] = filesize($file_data);
        $compress_data['original']['data'] = wp_get_attachment_metadata($attachment_id);

        if ($response == 'json') {
          wp_send_json_success();
        } else {
          if ($response == 'table') {

            $image     = wp_get_attachment_image_src($attachment_id, 'full');
            $file_name = basename($image[0]);

            // Not compressed
            $file_data              = get_attached_file($attachment_id);
            $uncompressed_value_num = $uncompressed_value = filesize($file_data);
            $uncompressed_value_num = $uncompressed_value_num / 1000;
            $uncompressed_value     = size_format($uncompressed_value, 2);

            $dimensions = wp_get_attachment_metadata($attachment_id);

            $output = '';
            $output .= '<div class="wps-ic-uncompressed">';
            $output .= '<img src="' . $logo_uncompressed . '" />';
            $output .= '<h5>Not Compressed</h5>';
            $output .= '</div>';

            $actions = '';

            $actions .= '<div class="wps-ic-media-actions">';
            $actions .= '<ul class="wps-ic-noncompressed-icon">';
            $actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
            $actions .= '<li class="wps-ic-size"><span>' . $dimensions['width'] . 'x' . $dimensions['height'] . '</span></li>';
            $actions .= '<li class="wps-ic-li-no-padding">';
            $actions .= '<div class="wps-ic-media-actions-toolbox">';
            $actions .= '<ul class="wps-ic-noncompressed-icon">';
            $actions .= '<li class="wps-ic-action">';

            $actions .= '<div class="btn-group">';
            $actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image="' . $image[0] . '" data-filename="' . $file_name . '" data-image_id="' . $attachment_id . '">Compress</button>';
            $actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
            $actions .= '<span class="caret"></span>';
            $actions .= '<span class="sr-only">Toggle Dropdown</span>';
            $actions .= '</button>';
            $actions .= '<ul class="dropdown-menu">';
            $actions .= '<li><a href="#wps-ic-compare' . $attachment_id . '" data-original="' . $attachment_id . '" data-compressed="' . $attachment_id . '" class="wps-ic-compare-single">Preview</a></li>';
            $actions .= '<li role="separator" class="divider"></li>';
            $actions .= apply_filters('wps_ic_pro_exclude', $attachment_id);
            $actions .= '</ul>';
            $actions .= '</div>';
            $actions .= '</li>';
            $actions .= '</ul>';
            $actions .= '</div>';

            $actions .= '<div id="wps-ic-compare' . $attachment_id . '" class="lightbox wp-ic-compare"></div>';

            wp_send_json_success(array('info' => $output, 'actions' => $actions, 'attachment_id' => $attachment_id));
          } else if ($response == 'complete') {

            $last_image = wp_get_attachment_image_src($attachment_id, 'full');

            $html = '';

            $html .= '<div class="wps-ic-sample-data" id="wps-ic-sample-data-' . $attachment_id . '">';

            //
            $html .= '<h3>Restore Complete</h3>';
            $html .= '<p>As long as you keep the backup original images setting on, you can restore your images back to original at any point if you want to recompress on different settings.</p>';

            // Image
            $html .= '<img src="' . $last_image[0] . '" id="wps-ic-sample-image" />';

            // wps_ic_sample_data
            $html .= '<a href="#"></a>';
            $html .= '</div>';

            wp_send_json_success(array('html' => $html));
          }
        }
      }
    }

    if ($response == 'json') {
      wp_send_json_error();
    }
  }


  public function nextgen_compress($fileurl, $filepath) {
    global $wps_ic;

    $compress_data = array();

    // Change PHP Limits
    $wps_ic->backup->change_limits();

    // Linode
    $send = wp_remote_get($this->api_url . '?image=' . $fileurl . '&apikey=' . $wps_ic::$api_key . '&force=true', array('timeout' => '30'));
    $body = wp_remote_retrieve_body($send);

    $body = json_decode($body, true);

    $body = $body['data'];

    if ( ! empty($body['compressed'])) {

      // Get file contents
      $send = wp_remote_get($body['compressed'], array('timeout' => '30'));
      $body = wp_remote_retrieve_body($send);

      // Save to same filename as old, and same path
      $savefile = fopen($filepath, 'w');
      fwrite($savefile, $body);
      fclose($savefile);
    }
  }


  public function preview($atts) {
    global $wps_ic;

    $stats    = $wps_ic->options->get_stats();
    $settings = $wps_ic->options->get_settings();

    $compress_data = array();

    // Change PHP Limits
    $wps_ic->backup->change_limits();

    // Get attachment ID
    $attach_id = sanitize_text_field($atts['attachment_id']);

    // Sanitize attachment filename
    $file = wp_get_attachment_image_src($attach_id, 'full');
    $file = $file[0];

    // Set compressing transient
    #set_transient('wps_ic_' . $attach_id, 'true', 60*5);

    // Resize larger images
    $resize_larger_images = '';
    if ($settings['resize_larger_images'] == '1') {
      $resize_larger_images = '&resize_max=' . $settings['resize_larger_images_width'];
    } else {
      $resize_larger_images = '&resize_max=0';
    }

    // Perserve Exif Data
    $preserve_exif = '&preserve_exif=0';
    if ($settings['preserve_exif'] == '1') {
      $preserve_exif = '&preserve_exif=1';
    }

    // Custom Quality
    if ($settings['use_custom_quality'] == '1') {
      $quality        = $settings['custom_quality'];
      $custom_quality = '&cq=' . $quality;
    }

    // API
    $send = wp_remote_get($this->api_url . '?image=' . $file . $custom_quality . '&q=' . $settings['optimization'] . $resize_larger_images . '' . $preserve_exif . '&apikey=' . $wps_ic::$api_key . '&force=true&bulk=true&attachment_ID=' . $attach_id,
                          array('timeout' => '120'));

    $body = wp_remote_retrieve_body($send);

    $body = json_decode($body, true);
    $body = $body['data'];

    if ( ! empty($body['compressed'])) {
      return array('uri' => $body['compressed']['full']['uri'], 'size' => $body['compressed']['full']['size']);
    }

    return false;
  }


  public function single($atts, $response) {
    global $wps_ic, $wpdb;

    $attachment_id = $atts['attachment_id'];

    // Update time
    $times                     = get_post_meta($attachment_id, 'wps_ic_times', true);
    $times['compress_started'] = microtime(true);
    update_post_meta($attachment_id, 'wps_ic_times', $times);

    $compress_data = array();
    $stats         = $wps_ic->options->get_stats();
    $settings      = $wps_ic->options->get_settings();

    // Get bulked files for compress background
    $bulk_compress_background = get_option(WPS_IC_BULK);

    if ( ! function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    // Is this image excluded?
    if (self::is_excluded($attachment_id)) {
      wp_send_json_success('excluded');
    }

    // Allowed type?
    if (self::is_allowed_type($attachment_id)) {
      wp_send_json_success('not allowed type');
    }

    // Get all image sizes
    $thumbs = array();
    $thumbs = self::get_all_thumbnails($attachment_id);

    // Is already compressed?
    #$backup = self::backup_exists($attachment_id);
    #$data   = get_post_meta($attachment_id, 'wps_ic_data', true);
    /*
        if ( ! empty($backup) && file_exists($backup[0]->path) && $data) {

          delete_post_meta($attachment_id, 'wps_ic_compressed');
          delete_post_meta($attachment_id, 'wps_ic_in_bulk');

          if ($response == 'none') {
          } elseif ($response == 'json') {
            $output  = '';
            $actions = '';

            $output .= '<div class="wps-ic-compressed">';
            $output .= '<img src="' . $logo_compressed . '" />';
            $output .= '<h5>Already Compressed</h5>';
            $output .= '</div>';

            $actions .= '<div class="wps-ic-media-actions">';
            $actions .= '<ul class="wps-ic-noncompressed-icon">';
            $actions .= '<li class="wps-ic-weight"><span>' . $compressed_value . '</span></li>';
            $actions .= '<li class="wps-ic-size"><span>' . $dimensions['width'] . 'x' . $dimensions['height'] . '</span></li>';
            $actions .= '<li class="wps-ic-li-no-padding">';
            $actions .= '<div class="wps-ic-media-actions-toolbox">';
            $actions .= '<ul class="wps-ic-noncompressed-icon">';
            $actions .= '<li class="wps-ic-action">';

            $actions .= '<div class="btn-group">';
            $actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image="' . $image[0] . '" data-filename="' . $file_name . '" data-image_id="' . $attach_id . '">Compress</button>';
            $actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
            $actions .= '<span class="caret"></span>';
            $actions .= '<span class="sr-only">Toggle Dropdown</span>';
            $actions .= '</button>';
            $actions .= '<ul class="dropdown-menu">';
            $actions .= '<li><a href="#wps-ic-compare' . $attach_id . '" data-original="' . $attach_id . '" data-compressed="' . $attach_id . '" class="wps-ic-compare-single">Compare Original vs Compressed</a></li>';
            $actions .= '<li role="separator" class="divider"></li>';
            $actions .= apply_filters('wps_ic_pro_exclude', $attach_id);
            $actions .= '</ul>';
            $actions .= '</div>';
            $actions .= '</li>';
            $actions .= '</ul>';
            $actions .= '</div>';

            $actions .= '</li>';
            $actions .= '</ul>';
            $actions .= '</div>';

            $actions .= '<div id="wps-ic-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
            $actions .= '<div id="wps-ic-pro-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
            wp_send_json_success();
          }
        }*/

    // Resize larger images
    $resize_larger_images = '';
    if ($settings['resize_larger_images'] == '1') {
      $resize_larger_images = '&resize_max=' . $settings['resize_larger_images_width'];
    } else {
      $resize_larger_images = '&resize_max=0';
    }

    // Perserve Exif Data
    $preserve_exif = '&preserve_exif=0';
    if ($settings['preserve_exif'] == '1') {
      $preserve_exif = '&preserve_exif=1';
    }

    // Custom Quality
    if ($settings['use_custom_quality'] == '1') {
      $quality        = $settings['custom_quality'];
      $custom_quality = '&cq=' . $quality;
    }

    // Priority?
    $priority = '';
    if ($settings['priority_compression'] == '1') {
      $priority = '&priority=' . WPS_IC_PRO_PRIORITY;
    }

    $wps_ic->log->write('Single CRON', 'Starting compress ' . $attachment_id, 'log.txt');
    $wps_ic->log->write('Single CRON',
                        $this->api_url . '?images=' . $thumbs . $custom_quality . $priority . '&q=' . $settings['optimization'] . $resize_larger_images . '' . $preserve_exif . '&apikey=' . $wps_ic::$api_key . '&force=true&attachment_ID=' . $attachment_id,
                        'log.txt');

    // API
    $send = wp_remote_get($this->api_url . '?images=' . $thumbs . $custom_quality . $priority . '&q=' . $settings['optimization'] . $resize_larger_images . '' . $preserve_exif . '&apikey=' . $wps_ic::$api_key . '&force=true&attachment_ID=' . $attachment_id,
                          array('timeout' => '120'));

    // Update time
    $times                   = get_post_meta($attachment_id, 'wps_ic_times', true);
    $times['compress_ended'] = microtime(true);
    update_post_meta($attachment_id, 'wps_ic_times', $times);
  }


  public function is_excluded($attachment_id) {
    $excluded = get_post_meta($attachment_id, 'wps_ic_exclude', true);
    if ( ! empty($excluded)) {

      // Get bulked files for compress background
      $bulk_queue = get_option(WPS_IC_BULK);

      // Remove from Bulk Compress Background
      unset($bulk_queue[ $attachment_id ]);
      update_option(WPS_IC_BULK, $bulk_queue);

      return true;
    }

    return false;
  }


  public function is_allowed_type($attachment_id) {
    $file_data = get_attached_file($attachment_id);
    $type      = wp_check_filetype($file_data);

    $allowed_types         = array();
    $allowed_types['jpg']  = 'jpg';
    $allowed_types['jpeg'] = 'jpeg';
    $allowed_types['gif']  = 'gif';
    $allowed_types['png']  = 'png';

    if ( ! in_array($type['ext'], $allowed_types)) {
      return true;
    }

    return false;
  }


  public function get_all_thumbnails($attachment_id) {
    $thumbs = array();
    $sizes  = get_intermediate_image_sizes();

    $thumbs['full'] = wp_get_attachment_image_src($attachment_id, 'full');
    $thumbs['full'] = $thumbs['full'][0];

    if (is_array($sizes)) {
      foreach ($sizes as $key => $size) {
        $thumbs[ $size ] = wp_get_attachment_image_src($attachment_id, $size);
        $thumbs[ $size ] = $thumbs[ $size ][0];
      }
    }

    return json_encode($thumbs);
  }


  public function backup_exists($attachment_id) {
    global $wpdb, $wps_ic;

    $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attachment_id);

    return $wpdb->get_results($backup);
  }


  public function single_bulk($atts, $response) {
    global $wps_ic, $wpdb;

    // Get attachment ID
    $attach_id = sanitize_text_field($atts['attachment_id']);

    // Change memory limits
    $wps_ic->backup->change_limits();

    // Have we stopped bulk?!?!
    if ( ! empty($atts['data']['status']) && $atts['data']['status'] == '-1') {
      delete_post_meta($attach_id, 'wps_ic_data');
      delete_post_meta($attach_id, 'wps_ic_times');

      $wps_ic->queue->remove_queue($attach_id);

      return false;
    }

    // Update time
    $times                   = get_post_meta($attach_id, 'wps_ic_times', true);
    $times['client_started'] = microtime(true);
    update_post_meta($attach_id, 'wps_ic_times', $times);

    // Old image data
    $file_data                    = get_attached_file($attach_id);
    $compress_data['old']['size'] = filesize($file_data);
    $compress_data['old']['data'] = wp_get_attachment_metadata($attach_id);
    update_post_meta($attach_id, 'wps_ic_data', $compress_data);

    // Update time
    $times                 = get_post_meta($attach_id, 'wps_ic_times', true);
    $times['client_ended'] = microtime(true);
    update_post_meta($attach_id, 'wps_ic_times', $times);

    // Update time
    $times                     = get_post_meta($attach_id, 'wps_ic_times', true);
    $times['compress_started'] = microtime(true);
    update_post_meta($attach_id, 'wps_ic_times', $times);

    $compress_data = get_post_meta($attach_id, 'wps_ic_data', true);
    $settings      = $wps_ic->options->get_settings();

    if ( ! function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    // Is already compressed?
    $data = get_post_meta($attach_id, 'wps_ic_data', true);
    if ( ! empty($data) && !empty($data['new']['size'])) {
      $wps_ic->queue->remove_compress_queue($attach_id);
      wp_send_json_success(array('msg' => 'ic_data_not_empty', 'data' => print_r($data, true)));
    }

    if ($data == 'not_able') {
      // Update compress data
      update_post_meta($attach_id, 'wps_ic_compressed', 'true');
      update_post_meta($attach_id, 'wps_ic_data', 'not_able');
      delete_post_meta($attach_id, 'wps_ic_in_bulk');
      delete_post_meta($attach_id, 'wps_ic_compressing');

      $wps_ic->queue->remove_compress_queue($attach_id);

      return $this->do_response($attach_id, 'table', '', $compress_data);

      //wp_send_json_success(array('msg' => 'ic_data_not_empty', 'data' => print_r($data, true)));
    }

    // Is this image excluded?
    if (self::is_excluded($attach_id)) {
      // Remove from Bulk Compress Background
      $wps_ic->queue->remove_queue($attach_id);
      wp_send_json_success('excluded');
    }

    // Is allowed type?
    if (self::is_allowed_type($attach_id)) {
      $wps_ic->queue->remove_queue($attach_id);
      wp_send_json_success('not allowed');
    }

    // Write Log
    $wps_ic->log->write('Started Compress Full Image ONLY', $attach_id);

    // Get all image sizes
    $thumbs = self::get_all_thumbnails($attach_id);

    // Is already compressed?
    $backup = $wpdb->prepare("SELECT * FROM " . $wps_ic->backup->backup_log . " 
                             WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1",
                             $attach_id);

    $backup = $wpdb->get_results($backup);

    if (empty($backup)) {

      // First Create Backup
      if ($settings['backup'] == '1') {
        $wps_ic->backup->local_backup($attach_id);
      }

    }

    // Resize larger images
    $resize_larger_images = '';
    if (!empty($settings['resize_larger_images']) && $settings['resize_larger_images'] == '1') {
      $resize_larger_images = '&resize_max=' . $settings['resize_larger_images_width'];
    } else {
      $resize_larger_images = '&resize_max=0';
    }

    // Perserve Exif Data
    $preserve_exif = '&preserve_exif=0';
    if (!empty($settings['preserve_exif']) && $settings['preserve_exif'] == '1') {
      $preserve_exif = '&preserve_exif=1';
    }

    // Custom Quality
    if (!empty($settings['use_custom_quality']) && $settings['use_custom_quality'] == '1') {
      $quality        = $settings['custom_quality'];
      $custom_quality = '&cq=' . $quality;
    }

    // Priority?
    $priority = '';
    if ($settings['priority_compression'] == '1') {
      $priority = '&priority=' . WPS_IC_PRO_PRIORITY;
    }

    // API
    $send = wp_remote_get($this->api_url . '?bulk=true&images=' . $thumbs . $custom_quality . $priority . '&q=' . $settings['optimization'] . $resize_larger_images . '' . $preserve_exif . '&apikey=' . $wps_ic::$api_key . '&force=true&attachment_ID=' . $attach_id,
                          array('timeout' => '120'));

    if (wp_remote_retrieve_response_code($send) == 200) {

      $body = wp_remote_retrieve_body($send);
      $body = json_decode($body, true);
      $body = $body['data'];

      if ( ! empty($body['compressed'])) {

        #if ($compress_data['old']['size'] > $body['compressed']['full']['size']) {

        // Get upload directory path
        $uploaddir = wp_upload_dir();

        // Write Log
        $wps_ic->log->write('Compress Single Full Image ONLY', '----');
        $wps_ic->log->write('Compress Single Full Image ONLY', print_r($compress_data, true));
        $wps_ic->log->write('Compress Single Full Image ONLY', '----');
        $wps_ic->log->write('Compress Single Full Image ONLY', 'Got remote response');

        if (empty($compress_data['old']['data'])) {
          $file_data                              = get_attached_file($attach_id);
          $imagesize                              = getimagesize($file_data);
          $compress_data['old']['size']           = filesize($file_data);
          $compress_data['old']['data']['width']  = $imagesize[0];
          $compress_data['old']['data']['height'] = $imagesize[0];
        }

        // Is new file smaller than old file?
        if ($body['compressed']['full']['size'] >= $compress_data['old']['size']) {

          $compress_data['new']['size']           = $compress_data['old']['size'];
          $compress_data['new']['data']['width']  = $body['compressed']['full']['dimensions'][0];
          $compress_data['new']['data']['height'] = $body['compressed']['full']['dimensions'][1];

          // Update compress data
          update_post_meta($attach_id, 'wps_ic_compressed', 'true');
          update_post_meta($attach_id, 'wps_ic_data', 'not_able');
          delete_post_meta($attach_id, 'wps_ic_in_bulk');
          delete_post_meta($attach_id, 'wps_ic_compressing');

          $wps_ic->queue->remove_compress_queue($attach_id);
        } else {

          // Write Log
          $wps_ic->log->write('Compress Single Full Image ONLY', 'Remote file fetching');

          foreach ($body['compressed'] as $size => $data) {

            $uploadfile = $uploaddir['path'] . '/' . basename($data['uri']);

            $wps_ic->log->write('Compress Single', print_r($uploadfile, true));
            $wps_ic->log->write('Compress Single', print_r($data['uri'], true));

            // Download remote file to temp
            $temp_file = download_url($data['uri'], 30);

            $wps_ic->log->write('Compress Single Full Image ONLY', $temp_file);

            if ($temp_file) {

              // Write Log
              $wps_ic->log->write('Compress Single Full Image ONLY', 'Remote file saving started');

              // First delete the current file
              clearstatcache();

              if (is_writable($uploadfile)) {
                // Everything OK; delete the file
                unlink($uploadfile);
              }

              if (file_exists($uploadfile)) {
                unlink($uploadfile);
              }

              // Move new file to old location/name
              copy($temp_file, $uploadfile);

              // Chmod new file to original file permissions
              @chmod($uploadfile, 0644);

              /// Add file to compressed table for stats
              $wpdb->insert($wpdb->prefix . 'ic_compressed', array(
                'created'       => current_time('mysql'),
                'attachment_ID' => $attach_id,
                'saved'         => ($compress_data['old']['size'] - $body['compressed']['full']['size']),
                'percent_saved' => 100 - ($body['compressed']['full']['size'] / $compress_data['old']['size']) * 100
              ));

              // Generate thumbnails
              $thumbnails = wp_remote_get(site_url('?secret_key=' . $wps_ic::$api_key . '&thumbnails=true&attachment_ID=' . $attach_id), array('sslverify' => false, 'timeout' => 0.1));

              $wps_ic->log->write('Compress Single Full Image ONLY', 'Remote file fetching ended');

              // Make thumb and/or update metadata
              #wp_update_attachment_metadata((int)$attach_id, wp_generate_attachment_metadata((int)$attach_id, $uploadfile));

              $wps_ic->log->write('Compress Single Full Image ONLY', 'Update attachment metadata');

              delete_post_meta($attach_id, 'wps_ic_compressing');
              delete_post_meta($attach_id, 'wps_ic_in_bulk');
              $wps_ic->queue->remove_compress_queue($attach_id);

            }

          }

          $compress_data['new']['size']           = $body['compressed']['full']['size'];
          $compress_data['new']['data']['width']  = $body['compressed']['full']['dimensions'][0];
          $compress_data['new']['data']['height'] = $body['compressed']['full']['dimensions'][1];

          $wps_ic->log->write('Compress Single Full Image ONLY', 'Compressed image smaller than original ! OK!!');

          // Trigger possible updates on CDN and other plugins
          $uploadfile = $uploaddir['path'] . '/' . basename($body['compressed']['full']['uri']);
          update_attached_file((int)$attach_id, $uploadfile);

          // Update compress data
          update_post_meta($attach_id, 'wps_ic_compressed', 'true');
          update_post_meta($attach_id, 'wps_ic_data', $compress_data);
          delete_post_meta($attach_id, 'wps_ic_in_bulk');
          delete_post_meta($attach_id, 'wps_ic_compressing');

          // Update time
          $times                   = get_post_meta($attach_id, 'wps_ic_times', true);
          $times['compress_ended'] = microtime(true);
          update_post_meta($attach_id, 'wps_ic_times', $times);

        }
      }

    }

    return $this->do_response($attach_id, $response, $body, $compress_data);
  }


  public function do_response($attach_id, $response, $body, $compress_data) {
    global $wps_ic, $wpdb;

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    $icon_compress = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress.png" alt="Compress" />';
    $icon_pro      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro.png" alt="Compare with Pro" />';
    $icon_compare  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare.png" alt="Compare" />';
    $icon_restore  = '<img src="' . WPS_IC_URI . 'assets/images/icon-restore.png" alt="Restore" />';

    $icon_compress_disabled = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress-disabled.png" alt="Compress Disabled" />';
    $icon_pro_disabled      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro-disabled.png" alt="Compare with Pro Disabled" />';
    $icon_compare_disabled  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-disabled.png" alt="Compare Disabled" />';

    if ($response == 'json') {
      wp_send_json_success(array('body' => $body, 'size' => size_format($compress_data['new'], 2) . ' (' . size_format($compress_data['old'], 2) . ')'));
    } else {
      if ($response == 'table') {
        $output = '';

        // Saved meta data
        $meta = get_post_meta($attach_id, 'wps_ic_data', true);
        $time = get_post_meta($attach_id, 'wps_ic_times', true);

        $client   = ($time['client_ended'] - $time['client_started']);
        $compress = ($time['compress_ended'] - $time['compress_started']);
        $time     = number_format($client + $compress, 1) . 's';

        if ($meta != 'not_able') {

          if (empty($meta['old']['data']['width'])) {
            $attachment_dimensions_old = 'Unreadable';
            $attachment_dimensions_new = 'Unreadable';
          } else {
            $attachment_dimensions_old = $meta['old']['data']['width'] . 'x' . $meta['old']['data']['height'];
            $attachment_dimensions_new = $meta['new']['data']['width'] . 'x' . $meta['new']['data']['height'];
          }

          // Compressed
          $file_data            = get_attached_file($attach_id);
          $compressed_value_num = $compressed_value = filesize($file_data);
          $compressed_value     = size_format($compressed_value, 2);

          if ( ! empty($meta['old']['size'])) {
            // Old size
            $uncompressed_value_num = $meta['old']['size'] / 1000;
            $uncompressed_value     = size_format($meta['old']['size'], 2);

            // Saved total
            $saved_total = $meta['old']['size'] - $compressed_value_num;
            $saved_total = size_format($saved_total, 2);

            // Saved
            $saved = ($compressed_value_num / ($uncompressed_value_num * 1000)) * 100;
            $saved = round(100 - $saved, 0) . '%';
          } else {
            $saved = $saved_total = $uncompressed_value = $uncompressed_value_num = 0;
          }

          $output = '';
          $output .= '<div class="wps-ic-compressed">';
          $output .= '<img src="' . $logo_compressed . '" />';
          $output .= '<h5>' . $saved . '</h5>';
          $output .= '<h5>' . $saved_total . '</h5>';
          $output .= '<h5>Saved</h5>';
          $output .= '</div>';

          $backup_path = get_option($wps_ic->backup->backup_log);

          $actions = '';

          $actions .= '<div class="wps-ic-media-actions">';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';

          $actions .= '<li class="wps-ic-weight"><span class="strike">' . $uncompressed_value . '</span></li>';
          $actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '</li>';

          $actions .= '<li class="wps-ic-size-compressed"><span>' . $compressed_value . '</span></li>';
          #$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '<div class="wps-ic-media-actions-toolbox">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '<li class="wps-ic-action">';

          $actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-image_id="' . $attach_id . '">Restore</button>';

          $actions .= '</li>';
          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '</div>';

        } else {
          // Compressed
          $file_data        = get_attached_file($attach_id);
          $compressed_value = filesize($file_data);
          $compressed_value = size_format($compressed_value, 2);
          $dimensions       = wp_get_attachment_metadata($attach_id);

          $image     = wp_get_attachment_image_src($attach_id, 'full');
          $file_name = basename($image[0]);

          $output  = '';
          $actions = '';

          $output .= '<div class="wps-ic-compressed">';
          $output .= '<img src="' . $logo_compressed . '" />';
          $output .= '<h5>Already Compressed</h5>';
          $output .= '</div>';

          $actions .= '<div class="wps-ic-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '<li class="wps-ic-weight"><span>' . $compressed_value . '</span></li>';
          $actions .= '<li class="wps-ic-size"><span>' . $dimensions['width'] . 'x' . $dimensions['height'] . '</span></li>';
          $actions .= '<li class="wps-ic-li-no-padding">';
          $actions .= '<div class="wps-ic-media-actions-toolbox">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '<li class="wps-ic-action">';

          $actions .= '<div class="btn-group">';
          $actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image="' . $image[0] . '" data-filename="' . $file_name . '" data-image_id="' . $attach_id . '">Compress</button>';
          $actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
          $actions .= '<span class="caret"></span>';
          $actions .= '<span class="sr-only">Toggle Dropdown</span>';
          $actions .= '</button>';
          $actions .= '<ul class="dropdown-menu">';
          $actions .= '<li><a href="#wps-ic-compare' . $attach_id . '" data-original="' . $attach_id . '" data-compressed="' . $attach_id . '" class="wps-ic-compare-single">Preview</a></li>';
          $actions .= '<li role="separator" class="divider"></li>';
          $actions .= apply_filters('wps_ic_pro_exclude', $attach_id);
          $actions .= '</ul>';
          $actions .= '</div>';
          $actions .= '</li>';
          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '</li>';
          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '<div id="wps-ic-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
          $actions .= '<div id="wps-ic-pro-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
        }

        wp_send_json_success(array('info' => $output, 'actions' => $actions, 'restore_path' => $wps_ic->backup->local_backup_path . $backup_path[ $attach_id ][0]['filename']));
      } else {
        if ($response == 'html') {

          // Saved meta data
          $meta = get_post_meta($attach_id, 'wps_ic_data', true);
          $time = get_post_meta($attach_id, 'wps_ic_times', true);

          $image_name    = wp_get_attachment_image_src($attach_id, 'full');
          $image         = wp_get_attachment_image_src($attach_id, 'medium');
          $compress_data = get_post_meta($attach_id, 'wps_ic_data', true);

          // Old File
          $backup = $wpdb->prepare("SELECT url FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attach_id);
          $backup = $wpdb->get_results($backup);

          // New image data
          $file_data            = get_attached_file($attach_id);
          $compress_data['new'] = filesize($file_data);

          // Get total proccessed
          $process_status = get_option(WPS_IC_BULK_PROCESS);

          $current_save = (($compress_data['new'] - $compress_data['old']['size']) / $compress_data['old']['size']);
          $current_save = $current_save * 100;
          $current_save = absint($current_save);
          $current_save = round($current_save, 0);

          // Compressed
          $file_data            = get_attached_file($attach_id);
          $compressed_value_num = $compressed_value = filesize($file_data);
          $compressed_value     = size_format($compressed_value, 2);

          if ( ! empty($meta['old']['size'])) {
            // Old size
            $uncompressed_value_num = $meta['old']['size'] / 1000;
            $uncompressed_value     = size_format($meta['old']['size'], 2);
          }

          // Stats
          $total_files = get_option(WPS_IC_STATS_BULK_TOTAL_FILES);
          //
          $files = get_option(WPS_IC_STATS_BULK_FILES);
          if ( ! $files) {
            $files = 0;
          }
          $files ++;
          update_option(WPS_IC_STATS_BULK_FILES, $files);
          //
          $savings = get_option(WPS_IC_STATS_BULK_SAVINGS);
          if ( ! $savings) {
            $savings = 0;
          }
          $savings = $savings + ($compress_data['old']['size'] - $compress_data['new']);
          update_option(WPS_IC_STATS_BULK_SAVINGS, $savings);
          //
          $avg = get_option(WPS_IC_STATS_BULK_AVG);
          if ( ! $avg) {
            $avg = array();
          }
          $avg[] = $current_save;

          $avg_data = 0;

          if ($avg) {
            foreach ($avg as $k => $v) {
              $avg_data += $v;
            }
            $avg_data = $avg_data / count($avg);
          }

          $avg_data = round($avg_data);
          update_option(WPS_IC_STATS_BULK_AVG, $avg);
          //
          $process_status['total_files']     = $total_files;
          $process_status['processed_files'] = $files;

          $client   = ($time['client_ended'] - $time['client_started']);
          $compress = ($time['compress_ended'] - $time['compress_started']);
          $time     = number_format($client + $compress, 1) . 's';

          if ($meta != 'not_able') {

            if (empty($meta['old']['data']['width'])) {
              $attachment_dimensions_old = 'Unreadable';
              $attachment_dimensions_new = 'Unreadable';
            } else {
              $attachment_dimensions_old = $meta['old']['data']['width'] . 'x' . $meta['old']['data']['height'];
              $attachment_dimensions_new = $meta['new']['data']['width'] . 'x' . $meta['new']['data']['height'];
            }
          }

          if ($meta == 'not_able') {
            // New image data
            $file_data                    = get_attached_file($attach_id);
            #$compress_data['new']         = filesize($file_data);
            #$compress_data['old']['size'] = filesize($file_data);

            $old_image = wp_get_attachment_image_src($attach_id, 'full');
            $old_image = $old_image[0];
            $old_size = filesize($file_data);
            $new_size = filesize($file_data);

          } else {

            if (!empty($backup[0]->url)) {
              $old_image = $backup[0]->url;
            } else {
              $old_image = wp_get_attachment_image_src($attach_id, 'full');
              $old_image = $old_image[0];
            }

            $old_size = $compress_data['old']['size'];
            $new_size = $compress_data['new'];
          }

          $output = '';
          $output .= '<h1 class="center wps-ic-big-heading">Just Compressed</h1>';

          $output .= '<div class="just-optimized-row">';

          $output .= '<div class="third">';
          $output .= '<h3>Before</h3>';
          $output .= '<div class="just-optimized-item wps-ic-before-container">';
          $output .= '<img src="' . $old_image . '"/>';
          $output .= '<h4>' . size_format($old_size, 2) . '</h4>';
          $output .= '</div>';
          $output .= '</div>';

          $output .= '<div class="third control-box">';
          $output .= '<h2 class="center">' . basename($image_name[0]) . '</h2>';
          $output .= '<div class="just-optimized-item logo-bulk">';
          $output .= '<img src="' . WPS_IC_URI . 'assets/images/logo-bulk.png" alt="WP Compress"/>';

          if ($current_save <= 0 || $meta == 'not_able') {
            $output .= '<h4>Already Compressed</h4>';
          } else {
            $output .= '<h4>' . $current_save . '% Saved</h4>';
          }

          $output .= '</div>';
          $output .= '</div>';

          $output .= '<div class="third">';
          $output .= '<h3>After</h3>';
          $output .= '<div class="just-optimized-item" wps-ic-after-container>';
          $output .= '<img src="' . $image[0] . '"/>';
          $output .= '<h4>' . size_format($new_size, 2) . '</h4>';
          $output .= '</div>';
          $output .= '</div>';

          $output .= '<div class="clearfix"></div>';

          $output .= '</div>';

          /**
           * Restore Button
           */
          $actions = '';

          $actions .= '<div class="wps-ic-media-actions">';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';

          $actions .= '<li class="wps-ic-weight"><span class="strike">' . $uncompressed_value . '</span></li>';
          $actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';
          $actions .= '<div class="wps-ic-half-media-actions">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '</li>';

          $actions .= '<li class="wps-ic-size-compressed"><span>' . $compressed_value . '</span></li>';
          #$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '<div class="wps-ic-media-actions-toolbox">';
          $actions .= '<ul class="wps-ic-noncompressed-icon">';
          $actions .= '<li class="wps-ic-action">';

          $actions .= '<button type="button" class="btn btn-info wps-ic-restore-single-demo" data-image_id="' . $attach_id . '">Restore</button>';

          $actions .= '</li>';
          $actions .= '</ul>';
          $actions .= '</div>';

          $actions .= '</div>';

          /**
           * Statistics
           */

          $stats = '';
          $stats .= '<div class="third">';
          $stats .= '<h3>' . $files . '/' . $total_files . '</h3>';
          $stats .= '<h5>Images Compressed</h5>';
          $stats .= '</div>';
          $stats .= '<div class="third">';
          $stats .= '<h3>' . size_format($savings, 2) . '</h3>';
          $stats .= '<h5>Total Savings</h5>';
          $stats .= '</div>';
          $stats .= '<div class="third last">';
          $stats .= '<h3>' . $avg_data . '%</h3>';
          $stats .= '<h5>Average Reduction</h5>';
          $stats .= '</div>';
          $stats .= '<div class="clearfix"></div>';

          #wp_send_json_success(array('html' => $output, 'status' => $process_status, 'stats' => $stats));
          return array('html' => $output, 'actions' => $actions, 'status' => $process_status, 'stats' => $stats);
        } else {
          if ($response == 'html-json') {
            $image_name    = wp_get_attachment_image_src($attach_id, 'full');
            $image         = wp_get_attachment_image_src($attach_id, 'medium');
            $compress_data = get_post_meta($attach_id, 'wps_ic_data', true);

            // Old File
            $backup = $wpdb->prepare("SELECT url FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attach_id);
            $backup = $wpdb->get_results($backup);

            // New image data
            $file_data            = get_attached_file($attach_id);
            $compress_data['new'] = filesize($file_data);

            // Get total proccessed
            $process_status = get_option(WPS_IC_BULK_PROCESS);

            // Write total saved
            $savings = get_option(WPS_IC_SAVINGS);

            $savings = $savings + ($compress_data['old']['size'] - $compress_data['new']);
            update_option(WPS_IC_SAVINGS, $savings);

            /**
             * Statistics
             */
            $stats = '';
            $stats .= '<div class="third">';
            $stats .= '<h3>' . $process_status['processed_files'] . '/' . $process_status['total_files'] . '</h3>';
            $stats .= '<h5>Images Compressed</h5>';
            $stats .= '</div>';
            $stats .= '<div class="third">';
            $stats .= '<h3>' . size_format($savings, 2) . '</h3>';
            $stats .= '<h5>Total Savings</h5>';
            $stats .= '</div>';
            $stats .= '<div class="third last">';
            $stats .= '<h3>80</h3>';
            $stats .= '<h5>Average Reduction</h5>';
            $stats .= '</div>';
            $stats .= '<div class="clearfix"></div>';

            wp_send_json_success(array(
                                   'before_img'  => $backup[0]->url,
                                   'before_size' => size_format($compress_data['old']['size'], 2),
                                   'after_img'   => $image[0],
                                   'after_size'  => size_format($compress_data['new'], 2),
                                   'status'      => $process_status,
                                   'stats'       => $stats
                                 ));
          } else {
            if ($response == 'bulk-json') {

              $image_name    = wp_get_attachment_image_src($attach_id, 'full');
              $image         = wp_get_attachment_image_src($attach_id, 'medium');
              $compress_data = get_post_meta($attach_id, 'wps_ic_data', true);

              // Old File
              $backup = $wpdb->prepare("SELECT url FROM " . $wps_ic->backup->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attach_id);
              $backup = $wpdb->get_results($backup);

              // New image data
              $file_data            = get_attached_file($attach_id);
              $compress_data['new'] = filesize($file_data);

              // Get total proccessed
              $process_status = get_option(WPS_IC_BULK_PROCESS);

              // Write total saved
              $savings = get_option(WPS_IC_SAVINGS);

              $savings = $savings + ($compress_data['old']['size'] - $compress_data['new']);
              update_option(WPS_IC_SAVINGS, $savings);

              /**
               * Statistics
               */
              $stats = '';
              $stats .= '<div class="third">';
              $stats .= '<h3>' . $process_status['processed_files'] . '/' . $process_status['total_files'] . '</h3>';
              $stats .= '<h5>Images Compressed</h5>';
              $stats .= '</div>';
              $stats .= '<div class="third">';
              $stats .= '<h3>' . size_format($savings, 2) . '</h3>';
              $stats .= '<h5>Total Savings</h5>';
              $stats .= '</div>';
              $stats .= '<div class="third last">';
              $stats .= '<h3>80</h3>';
              $stats .= '<h5>Average Reduction</h5>';
              $stats .= '</div>';
              $stats .= '<div class="clearfix"></div>';

              $array = array(
                'before_img'  => $backup[0]->url,
                'before_size' => size_format($compress_data['old']['size'], 2),
                'after_img'   => $image[0],
                'after_size'  => size_format($compress_data['new'], 2),
                'status'      => $process_status,
                'stats'       => $stats
              );

              return $array;
            }
          }
        }
      }
    }
  }

  public
  function exclude(
    $atts, $response = 'html'
  ) {
    global $wps_ic, $wpdb;

    $attachment_id = $atts['attachment_id'];
    $exclude       = get_post_meta($attachment_id, 'wps_ic_exclude', true);

    $logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';
    $logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
    $logo_excluded     = WPS_IC_URI . 'assets/images/excluded.png';

    $icon_compress = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress.png" alt="Compress" />';
    $icon_pro      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro.png" alt="Compare with Pro" />';
    $icon_compare  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare.png" alt="Compare" />';
    $icon_restore  = '<img src="' . WPS_IC_URI . 'assets/images/icon-restore.png" alt="Restore" />';

    $icon_compress_disabled = '<img src="' . WPS_IC_URI . 'assets/images/icon-compress-disabled.png" alt="Compress Disabled" />';
    $icon_pro_disabled      = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-pro-disabled.png" alt="Compare with Pro Disabled" />';
    $icon_compare_disabled  = '<img src="' . WPS_IC_URI . 'assets/images/icon-compare-disabled.png" alt="Compare Disabled" />';

    if (empty($exclude)) {
      update_post_meta($attachment_id, 'wps_ic_exclude', 'true');
      #$output = '<a href="#wp-ic-exclude-' . $id . '" data-attachment_id="' . $id . '" class="button button-primary wps-ic-pro-exclude">Include</a>';

      $output = '';
      $output .= '<div class="wps-ic-excluded">';
      $output .= '<img src="' . $logo_excluded . '" />';
      $output .= '<h5>Excluded</h5>';
      $output .= '</div>';

      $actions = '';
      $actions .= '<div class="wps-ic-media-actions-toolbox">';
      $actions .= '<ul class="wps-ic-include">';
      $actions .= '<li class="no-padding">';
      #$actions .= '<a href="#" title="Compress Disabled" class="wps-ic-button wps-ic-action-disabled" data-image_id="' . $attachment_id . '">' . $icon_compress_disabled . '</a>';
      $actions .= apply_filters('wps_ic_pro_include', $attachment_id);
      #$actions .= '<a href="#wps-ic-compare' . $attachment_id . '" title="Compare Disabled" class="wps-ic-button wps-ic-action-disabled" data-original="' . $attachment_id . '" data-compressed="' . $attachment_id . '">' . $icon_compare_disabled . '</a>';
      $actions .= '</li>';
      $actions .= '</ul>';
      $actions .= '</div>';

    } else {
      delete_post_meta($attachment_id, 'wps_ic_exclude');
      #$output = '<a href="#wp-ic-exclude-' . $id . '" data-attachment_id="' . $id . '" class="button button-primary wps-ic-pro-exclude">Exclude</a>';

      // Not compressed
      $file_data              = get_attached_file($attachment_id);
      $uncompressed_value_num = $uncompressed_value = filesize($file_data);
      $uncompressed_value_num = $uncompressed_value_num / 1000;
      $uncompressed_value     = size_format($uncompressed_value, 2);

      $dimensions = wp_get_attachment_metadata($attachment_id);
      $image      = wp_get_attachment_image_src($attachment_id, 'full');
      $file_name  = basename($image[0]);

      $output = '';
      $output .= '<div class="wps-ic-uncompressed">';
      $output .= '<img src="' . $logo_uncompressed . '" />';
      $output .= '<h5>Not Compressed</h5>';
      $output .= '</div>';

      $actions = '';
      $actions .= '<div class="wps-ic-media-actions">';
      $actions .= '<ul class="wps-ic-noncompressed-icon">';
      $actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
      $actions .= '<li class="wps-ic-size"><span>' . $dimensions['width'] . 'x' . $dimensions['height'] . '</span></li>';
      $actions .= '<li class="wps-ic-li-no-padding">';
      $actions .= '<div class="wps-ic-media-actions-toolbox">';
      $actions .= '<ul class="wps-ic-noncompressed-icon">';
      $actions .= '<li class="wps-ic-action">';

      $actions .= '<div class="btn-group">';
      $actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image="' . $image[0] . '" data-filename="' . $file_name . '" data-image_id="' . $attachment_id . '">Compress</button>';
      $actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
      $actions .= '<span class="caret"></span>';
      $actions .= '<span class="sr-only">Toggle Dropdown</span>';
      $actions .= '</button>';
      $actions .= '<ul class="dropdown-menu">';
      $actions .= '<li><a href="#wps-ic-compare' . $attachment_id . '" data-original="' . $attachment_id . '" data-compressed="' . $attachment_id . '" class="wps-ic-compare-single">Preview</a></li>';
      $actions .= '<li role="separator" class="divider"></li>';
      $actions .= apply_filters('wps_ic_pro_exclude', $attachment_id);
      $actions .= '</ul>';
      $actions .= '</div>';
      $actions .= '</li>';
      $actions .= '</ul>';
      $actions .= '</div>';

      $actions .= '<div id="wps-ic-compare' . $attachment_id . '" class="lightbox wp-ic-compare"></div>';

    }

    wp_send_json_success(array('info' => $output, 'actions' => $actions));
  }

}