<?php


/**
 * Class - Stats
 */
class wps_ic_stats extends wps_ic {


  public function __construct() {
    if (is_admin()) {
      global $wpdb;
    }
  }


  public function get() {
    $bytes = get_option(WPS_IC_STATS_BYTES);
    $files = get_option(WPS_IC_STATS_FILES);
    $avg = get_option(WPS_IC_STATS_AVG_REDUCTION);

    if (!empty($avg)) {
      $avg_savings = 0;
      foreach ($avg as $key => $value) {
        $avg_savings += $value;
      }

      $avg_savings = $avg_savings/count($avg);
    }

    return array('bytes' => size_format($bytes, 2), 'files' => $files, 'avg' => $avg_savings);
  }


  public function update($old_bytes = 0, $new_bytes = 0, $action = 'add') {

    $bytes     = round($old_bytes - $new_bytes, 0);
    $reduction = ($new_bytes / $old_bytes) * 100;
    $reduction = 100 - $reduction;
    $reduction = round($reduction, 0);

    if ($action == 'add') {
      $this->add_file();
      $this->add_bytes($bytes);
      $this->average_reduction($reduction);
    } else {
      $this->remove_file();
      $this->remove_bytes($bytes);
      $this->remove_avg_reduction($reduction);
    }
  }


  public function add_file() {
    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $option = get_option(WPS_IC_STATS_FILES);

    if ( ! $option) {
      $option = 0;
    }

    $option = $option + 1;
    update_option(WPS_IC_STATS_FILES, $option);
  }


  public function add_bytes($amount = 0) {
    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    if ($amount == 0) {
      return false;
    }

    $option = get_option(WPS_IC_STATS_BYTES);
    $option = $option + $amount;
    update_option(WPS_IC_STATS_BYTES, $option);
  }


  public function average_reduction($reduction) {
    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $option = get_option(WPS_IC_STATS_AVG_REDUCTION);

    if ( ! $option || $option == '' || $option == 0 || $option == '0') {
      $option = array();
    }

    $option[] = $reduction;
    update_option(WPS_IC_STATS_AVG_REDUCTION, $option);
  }


  public function remove_file() {
    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $option = get_option(WPS_IC_STATS_FILES);

    if ( ! $option) {
      $option = 0;
    }

    $option = $option - 1;
    update_option(WPS_IC_STATS_FILES, $option);
  }


  public function remove_bytes($amount = 0) {

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    if ($amount == 0) {
      return false;
    }

    $option = get_option(WPS_IC_STATS_BYTES);
    $option = $option - $amount;
    update_option(WPS_IC_STATS_BYTES, $option);
  }


  public function remove_avg_reduction($reduction) {
    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $option = get_option(WPS_IC_STATS_AVG_REDUCTION);

    if ( ! empty($option)) {
      foreach ($option as $key => $value) {
        if ($value == $reduction) {
          unset($option[ $key ]);
          break;
        }
      }

      update_option(WPS_IC_STATS_AVG_REDUCTION, $option);
    }
  }


}